# NATools - Package with parallel corpora tools
# Copyright (C) 2002-2004  Alberto Simes
#
# This package is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

package NAT::Client;

use 5.006;
use strict;
use warnings;
use CGI qw/:standard/;
require Exporter;

use locale;
use NAT;
use Data::Dumper;
use IO::Socket;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw();

our $VERSION = '0.03';


sub new {
  my $class = shift;
  my $self = { PeerAddr => '127.0.0.1',
	       PeerPort => '4000',
	       Proto    => 'tcp' };

  $self = {%$self, @_};

  if (exists($self->{local})) {
    NAT::corpus_info_open($self->{local});
  }

  if (exists($self->{localDumper})) {
    our ($DIC1, $DIC2);

    die "File not found." unless -f $self->{localDumper};

    do $self->{localDumper};

    $self->{d1} = $DIC1;
    $self->{d2} = $DIC2;
  }

  return bless $self => $class
}


sub list {
  local $/ = "\n";
  my $self = shift;
  my $data;

  return undef if exists $self->{localDumper};

  my $sock = new IO::Socket::INET ( %$self );
  die "Socket could not be created. Reason: $!\n" unless $sock;

  print $sock "LIST\n";
  my $nr = <$sock>;
  while($nr) {
    my $corpus = <$sock>;
    $corpus =~ m!^\[(\d+)\]\s(.*)!;
    $data->{$2} = { id => $1 };
    $nr--;
  }

  $sock->close;

  for (keys %$data) {
    $data->{$_}{source} = $self->attribute({crp=>$data->{$_}{id}}, "source-language");
    $data->{$_}{target} = $self->attribute({crp=>$data->{$_}{id}}, "target-language");
  }

  return $data;
}

sub set_corpus {
  my ($self,$crp) = @_;
  $crp||=1;
  $self->{select} = $crp;
  return $self;
}

sub ptd {
  my $self = shift;

  my $conf;
  $conf = shift if (ref($_[0]) eq "HASH");
  $conf->{crp} ||= $self->{select} || 1;
  $conf->{direction} = "~>" unless $conf->{direction} && $conf->{direction} eq "<~";

  my $word = shift;

  if (exists($self->{localDumper}) && $self->{localDumper}) {

    my $dir = $conf->{direction} eq "~>" ? "d1" : "d2";

    return undef unless exists $conf->{$dir}{$word};
    my ($o, %dic) = ($conf->{$dir}{$word}{count}, %{$conf->{$dir}{$word}{trans}});
    return [$o,\%dic];


  } elsif (exists($self->{local}) && $self->{local}) {
    my $dir = $conf->{direction} eq "~>" ? 1 : -1;

    my $x = NAT::corpus_info_ptd_by_word($dir, $word);
    return undef unless $x;
    my ($o,%dic) =  @$x;

    return [$o,\%dic];

  }  else {

    local $/ = "\n";

    my $sock = new IO::Socket::INET ( %$self );
    die "Socket could not be created. Reason: $!\n" unless $sock;

    print $sock "$conf->{direction} $conf->{crp} $word\n";
    my $occ = <$sock>;
    chomp($occ) if $occ;
    return undef unless $occ =~ m!^\d+$!;

    my $dic = {};
    my $trans = <$sock>;
    chomp($trans) if $trans;
    while($trans && $trans !~ /^\*\* .* \*\*$/) {
      $trans =~ m!^(\d+\.\d+)\s(\S+)!;
      $dic->{$2} = $1;

      $trans = <$sock>;
      chomp($trans) if $trans;
    }
    close ($sock);

    return [$occ, $dic];
  }
}


sub attribute {
  local $/ = "\n";

  my $self = shift;
  my $conf;
  $conf = shift if (ref($_[0]) eq "HASH");
  $conf->{crp} ||= $self->{select};

  my $var = shift;

  return undef if exists $self->{localDumper};

  my $sock = new IO::Socket::INET ( %$self );
  die "Socket could not be created. Reason: $!\n" unless $sock;

  print $sock "? $conf->{crp} $var\n";
  my $value = <$sock>;

  chomp($value) if $value;
  close ($sock);
  return "" unless $value;
  return $value;
}


sub conc {
  local $/ = "\n";

  my $self = shift;

  my $conf;
  $conf = shift if (ref($_[0]) eq "HASH");
  $conf->{crp} ||= $self->{select};
  $conf->{crp} ||= 1;
  $conf->{direction} = "->" unless $conf->{direction};
  $conf->{count} ||= 20;

  return undef if exists $self->{localDumper};

  my $sock = new IO::Socket::INET ( %$self );
  die "Socket could not be created. Reason: $!\n" unless $sock;

  my $left = lc(shift());

  my $count = $conf->{count};

  if ($conf->{direction} eq "<->" or
      $conf->{direction} eq "<=>") {
    $left .= " $conf->{direction} ".lc(shift());
  }

  print $sock "$conf->{direction} $conf->{crp} $left #$count\n";
  my @r = ();
  my $b1 = <$sock>;
  chomp($b1) if $b1;
  while($b1 && $b1 !~ /^\*\* .* \*\*$/) {
    my $rank = -1;

    if ($b1 =~ m!^\%\ (\d+\.\d+)$!) {
      $rank = $1;
      $b1 = <$sock>;
      chomp($b1) if $b1;
    }

    my $b2 = <$sock>;
    chomp($b2) if $b2;

    if ($rank >= 0) {
      push (@r, [$b1, $b2, $rank]);
    } else {
      push (@r, [$b1, $b2]);
    }


    $b1 = <$sock>;
    chomp($b1) if $b1;
  }
  close($sock);
  return \@r
}


sub my_header {

  my $JSCRIPT = shift || undef;

  my $CSS = <<EOCSS;
 a.link {
   color: #000000;
   text-decoration: none;
 }
 a.link:hover {
   text-decoration: underline;
 }
 form.main {
   text-align: center;
   background-color: #ffffdd;
   border-bottom: solid 1px #000000;
   padding: 3px;
   margin: 0px;
 }
 input {
   border: solid 1px #000000;
 }
 input[type=submit] {
   background-color: #eeeeee;
 }
 input[type=submit]:hover {
   border: solid 1px #ff7700;
   color: #ff0000;
   background-color: #ffff88;
 }
 input[type=text] {
   padding-left: 3px;
   padding-right: 3px;
 }
 th {
   background-color: #ffeedd;
   color: #ff7700;
   border-bottom: solid 1px #ff7700;
 }
 table.results {
   border: solid 1px #990000;
   margin-left: 5px;
   margin-right: 5px;
   padding: 0px;
 }
 td.entry {
   border-right: solid 1px #999999;
   padding: 3px;
 }
 h1 {
   border-bottom: solid 1px #000000;
   margin: 0px;
   padding: 3px;
   padding-top: 5px;
   background-color: wheat;
   text-align: center;
   font-size: 140%;
   color: #990000;
 }
 address {
   margin-right: 4px;
   font-size: 80%;
   color #999999;
   text-align: right;
   padding-top: 3px;
 }
 span {
   font-weight: bold;
 }
 div.hlpbt {
   float: right;
   border: solid 1px #000000;
   color: #000000;
   background-color: #eeeeee;
   padding: 2px;
   padding-left: 12px;
   padding-right: 12px;
   margin: 4px;
 }
 div.hlpbt:hover {
   float: right;
   border: solid 1px #ff7700;
   color: #ff0000;
   background-color: #ffff88;
   cursor: default;
 }
 body {
   margin: 0px;
 }
EOCSS

  my %conf = (
	      -title => "NATools Corpora Query Interface",
	      -style => {-verbatim => $CSS }
	     );

  $conf{-script} = $JSCRIPT if $JSCRIPT;

  return header,start_html(%conf);
}

sub close_window {
  return <<"EOT";
 <div class="hlpbt" onClick="window.close();">Close</div><br/>
EOT
}

sub my_footer {
  return address("&copy;2005 NATools"),end_html;
}


1;
__END__

=head1 NAME

NAT::Client - ...

=head1 SYNOPSIS

  use NAT::Client;

  $lex = NAT::Client::query("dog");

  $dic = NAT::Client::query_dictionary("word");

  $var = NAT::Client::query_variable("variable");

=head1 QUERY FUNCTIONS

This module includes functions to query the NATools server. By default
it queries the server in the localhost, port 4000. All the query
functions can receive an optional hash reference as first argument
with configuration information.

All the query functions share these same configuration values:

=over 4

=item PeerAddr

The IP address where the server is running on. Defaults to 127.0.0.1.

=item PeerPort

The port to be used in the connection. Defaults to 4000.

=back

=head2 C<query_variable>

This function is used to query the configuration file of the aligned
corpora. The configuration file just relates configuration parameter
names with their values. The function receive the optional configuration
hash reference and the variable name to be consulted.

 NAT::Client::query_variable( "name" );

The function returns the empty string if no value is found.

=head2 C<query_dictionary>

This function is used to query probabilistic translation dictionaries.

=head2 C<query>

=head1 OTHER FUNCTIONS

=head2 C<my_header>

  NAT::Client::my_header( "javascript-code" );

=head2 C<my_footer>

=head1 SEE ALSO

See perl(1) and NATools documentation.

=head1 AUTHOR

Alberto Manuel Brandao Simoes, E<lt>albie@alfarrabio.di.uminho.ptE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2002-2005 by Natura Project
http://natura.di.uminho.pt

This library is free software; you can redistribute it and/or modify
it under the GNU General Public License 2, which you should find on
parent directory. Distribution of this module should be done including
all NATools package, with respective copyright notice.

=cut
